@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #  bln.py - a easyballoon compatible Saori module for ninix
-#  Copyright (C) 2002-2011 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2002-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -19,10 +19,13 @@ import random
 import math
 import time
 import sys
+import logging
 
-import gtk
-import glib
-import pango
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GLib
+from gi.repository import Pango
+from gi.repository import PangoCairo
 import cairo
 
 import ninix.pix
@@ -45,12 +48,16 @@ class Saori(SAORI):
 
     def setup(self):
         self.blns = self.read_bln_txt(self.dir)
-        return 1 if self.blns else 0
+        if self.blns:
+            self.__sakura.attach_observer(self)
+            return 1
+        else:
+            return 0
 
     def read_bln_txt(self, dir):
         blns = {}
         try:
-            with open(os.path.join(dir, 'bln.txt'), 'r') as f:
+            with open(os.path.join(dir, b'bln.txt'), 'r', encoding='CP932') as f:
                 data = {}
                 name = ''
                 for line in f:
@@ -81,13 +88,22 @@ class Saori(SAORI):
     def finalize(self):
         for name in self.blns:
             data, bln = self.blns[name]
-            for bln_id in bln.keys():
+            for bln_id in list(bln.keys()):
                 if bln[bln_id]:
                     bln[bln_id].destroy()
                     del bln[bln_id]
         self.blns = {}
+        self.__sakura.detach_observer(self)
         return 1
 
+    def observer_update(self, event, args): ## FIXME
+        if event == 'set scale':
+            for name in self.blns:
+                data, bln = self.blns[name]
+                for bln_id in bln.keys():
+                    if bln[bln_id]:
+                        bln[bln_id].reset_scale()
+
     def execute(self, argument):
         if not argument:
             return self.RESPONSE[400]
@@ -128,13 +144,16 @@ class Saori(SAORI):
         elif name == 'clear':
             for name in self.blns:
                 data, bln = self.blns[name]
-                if bln_id in bln and \
-                   bln[bln_id].get_state() != 'orusuban':
+                new_bln = {}
+                for bln_id in bln:
+                    if bln[bln_id].get_state() == 'orusuban':
+                        new_bln[bln_id] = bln[bln_id]
+                        continue
                     bln[bln_id].destroy()
-                    del bln[bln_id]
-                    self.blns[name] = (data, {})
+                self.blns[name] = (data, new_bln)
         return None
 
+
 class Balloon:
 
     def __init__(self, sakura, dir, data,
@@ -143,18 +162,24 @@ class Balloon:
         self.__sakura = sakura
         self.name = name
         self.id = bln_id
+        self.timeout_id = None
         self.data = data # XXX
-        self.window = ninix.pix.TransparentWindow(type=gtk.WINDOW_POPUP)
+        self.window = ninix.pix.TransparentWindow()
+        self.window.set_focus_on_map(False)
+        self.window.set_title(name)
+        self.window.set_skip_taskbar_hint(True)
+        self.window.set_decorated(False)
+        self.window.set_resizable(False)
         self.window.connect('delete_event', self.delete)
         self.position = data.get('position', 'sakura')
         left, top, scrn_w, scrn_h = ninix.pix.get_workarea()
         # -1: left, 1: right
         if self.position == 'sakura':
             s0_x, s0_y, s0_w, s0_h = self.get_sakura_status('SurfaceSakura')
-            self.direction = -1 if s0_x + s0_w / 2 > left + scrn_w / 2 else 1
+            self.direction = -1 if s0_x + s0_w // 2 > left + scrn_w // 2 else 1
         elif self.position == 'kero':
             s1_x, s1_y, s1_w, s1_h = self.get_sakura_status('SurfaceKero')
-            self.direction = -1 if s1_x + s1_w / 2 > left + scrn_w / 2 else 1
+            self.direction = -1 if s1_x + s1_w // 2 > left + scrn_w // 2 else 1
         else:
             self.direction = 1 # XXX
         if self.position in ['sakura', 'kero']:
@@ -167,62 +192,52 @@ class Balloon:
         if skin is None:
             self.destroy()
             return
-        path = os.path.join(self.dir, skin.replace('\\', '/'))
+        path = os.path.join(self.dir, os.fsencode(skin.replace('\\', '/')))
         try:
-            balloon_pixbuf = ninix.pix.create_pixbuf_from_file(path)
+            balloon_surface = ninix.pix.create_surface_from_file(path)
         except:
             self.destroy()
             return
-        self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
-        w = balloon_pixbuf.get_width()
-        h = balloon_pixbuf.get_height()
-        w = max(8, int(w * self.scale / 100))
-        h = max(8, int(h * self.scale / 100))
-        balloon_pixbuf = balloon_pixbuf.scale_simple(
-            w, h, gtk.gdk.INTERP_BILINEAR)
-        mask_pixmap = gtk.gdk.Pixmap(None, w, h, 1)
-        balloon_pixbuf.render_threshold_alpha(
-            mask_pixmap, 0, 0, 0, 0, w, h, 1)
-        self.x, self.y = self.get_coordinate(w, h)
+        self.balloon_surface = balloon_surface
+        w = balloon_surface.get_width()
+        h = balloon_surface.get_height()
+        self.x = self.y = 0
         if 'offset.x' in data:
-            self.x += self.direction * int(int(data['offset.x']) * self.scale / 100)
+            self.x += self.direction * int(data['offset.x'])
         if 'offset.y' in data:
-            self.y += int(int(data['offset.y']) * self.scale / 100)
+            self.y += int(data['offset.y'])
         if 'offset.random' in data:
-            self.x += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
-            self.y += int(int(data['offset.random']) * random.randrange(-1, 2) * self.scale / 100)
-        self.x += self.direction * int(offset_x * self.scale / 100)
-        self.y += int(offset_y * self.scale / 100)
-        self.window.move(self.x, self.y)
-        gtk.gdk.flush() # XXX
-        self.left = int(int(data.get('disparea.left', 0)) * self.scale / 100)
-        self.right = int(int(data.get('disparea.right', w)) * self.scale / 100)
-        self.top = int(int(data.get('disparea.top', 0)) * self.scale / 100)
-        self.bottom = int(int(data.get('disparea.bottom', h)) * self.scale / 100)
-        if self.window.is_composited():
-            self.window.input_shape_combine_mask(mask_pixmap, 0, 0)
-        else:
-            self.window.shape_combine_mask(mask_pixmap, 0, 0)
-        self.current_mask = mask_pixmap
-        self.current_balloon_pixbuf = balloon_pixbuf
+            self.x += int(data['offset.random']) * random.randrange(-1, 2)
+            self.y += int(data['offset.random']) * random.randrange(-1, 2)
+        self.x += self.direction * int(offset_x)
+        self.y += int(offset_y)
+        self.action_x = 0
+        self.action_y = 0
+        self.vx = 0
+        self.vy = 0
+        self.left = int(data.get('disparea.left', 0))
+        self.right = int(data.get('disparea.right', w))
+        self.top = int(data.get('disparea.top', 0))
+        self.bottom = int(data.get('disparea.bottom', h))
         self.script = None
-        self.darea = gtk.DrawingArea()
-        self.darea.set_events(gtk.gdk.EXPOSURE_MASK|
-                              gtk.gdk.BUTTON_PRESS_MASK|
-                              gtk.gdk.BUTTON_RELEASE_MASK|
-                              gtk.gdk.POINTER_MOTION_MASK|
-                              gtk.gdk.LEAVE_NOTIFY_MASK)
-        self.darea.connect('expose_event', self.redraw)
+        self.darea = Gtk.DrawingArea()
+        self.darea.set_events(Gdk.EventMask.EXPOSURE_MASK|
+                              Gdk.EventMask.BUTTON_PRESS_MASK|
+                              Gdk.EventMask.BUTTON_RELEASE_MASK|
+                              Gdk.EventMask.POINTER_MOTION_MASK|
+                              Gdk.EventMask.LEAVE_NOTIFY_MASK)
+        self.darea.connect('draw', self.redraw)
         self.darea.connect('button_press_event', self.button_press)
         self.darea.connect('button_release_event', self.button_release)
         self.darea.connect('motion_notify_event', self.motion_notify)
         self.darea.connect('leave_notify_event', self.leave_notify)
-        self.darea.set_size_request(w, h)
         self.darea.show()
         self.window.add(self.darea)
         self.darea.realize()
         self.window.realize()
-        self.window.window.set_back_pixmap(None, False)
+        self.window.get_window().set_background_pattern(None) # XXX
+        self.set_skin()
+        self.set_position()
         self.layout = None
         if text != 'noscript' and \
            (self.right - self.left) and (self.bottom - self.top):
@@ -232,21 +247,19 @@ class Balloon:
                 fontcolor_g = int(data['font.color'][2:4], 16) /255.0
                 fontcolor_b = int(data['font.color'][4:6], 16) / 255.0
                 self.fontcolor = (fontcolor_r, fontcolor_g, fontcolor_b)
+            else:
+                self.fontcolor = (0.0, 0.0, 0.0) # XXX
             default_font_size = 12 # for Windows environment
-            font_size = int(data.get('font.size', default_font_size))
-            font_size = font_size * 3 / 4 # convert from Windows to GTK+
-            font_size = font_size * self.scale / 100 * pango.SCALE
-            self.layout = pango.Layout(self.darea.get_pango_context())
-            self.font_desc = pango.FontDescription()
+            self.font_size = int(data.get('font.size', default_font_size))
+            self.layout = Pango.Layout(self.darea.get_pango_context())
+            self.font_desc = Pango.FontDescription()
             self.font_desc.set_family('Sans') # FIXME
-            self.font_desc.set_size(font_size)
             if data.get('font.bold') == 'on':
-                self.font_desc.set_weight(pango.WEIGHT_BOLD)
-            self.layout.set_font_description(self.font_desc)
-            self.layout.set_wrap(pango.WRAP_CHAR)
-            self.layout.set_width((self.right - self.left) * 1024)
-        self.slide_vx = int(int(data.get('slide.vx', 0)) * self.scale / 100)
-        self.slide_vy = int(int(data.get('slide.vy', 0)) * self.scale / 100)
+                self.font_desc.set_weight(Pango.Weight.BOLD)
+            self.layout.set_wrap(Pango.WrapMode.CHAR)
+            self.set_layout()
+        self.slide_vx = int(data.get('slide.vx', 0))
+        self.slide_vy = int(data.get('slide.vy', 0))
         self.slide_autostop = int(data.get('slide.autostop', 0))
         if data.get('action.method') in ['sinwave', 'vibrate']:
             action = data['action.method']
@@ -284,16 +297,9 @@ class Balloon:
         self.nooverlap = int('nooverlap' in data)
         self.talking = self.get_sakura_is_talking()
         self.move_notify_time = time.time()
-        self.timeout_id = None
         self.visible = 0
         self.x_root = None
         self.y_root = None
-        self.action_x = 0
-        self.action_y = 0
-        self.vx = 0
-        self.vy = 0
-        self.text_pos_x = self.left
-        self.text_pos_y = self.top
         self.processed_script = None
         self.processed_text = ''
         self.text = ''
@@ -302,19 +308,58 @@ class Balloon:
         self.script_parser = ninix.script.Parser(error='loose')
         try:
             self.processed_script = self.script_parser.parse(self.script)
-        except ninix.script.ParserError, e:
+        except ninix.script.ParserError as e:
             self.processed_script = None
-            print '-' * 50
-            print e
-            print self.script                                    
-        self.timeout_id = glib.timeout_add(10, self.do_idle_tasks)
+            logging.error('-' * 50)
+            logging.error(e)
+            logging.error(self.script.encode('utf-8'))
+        self.timeout_id = GLib.timeout_add(10, self.do_idle_tasks)
 
+    def set_position(self):
+        if self.window is None:
+            return
+        new_x = self.base_x + int((self.x + self.action_x + self.vx) * self.scale / 100)
+        new_y = self.base_y + int((self.y + self.action_y + self.vy) * self.scale / 100)
+        self.window.resize_move(new_x, new_y)
+
+    def set_skin(self):
+        if self.window is None:
+            return
+        self.scale = self.get_sakura_status('SurfaceScale') ## FIXME
+        w = self.balloon_surface.get_width()
+        h = self.balloon_surface.get_height()
+        w = max(8, int(w * self.scale / 100))
+        h = max(8, int(h * self.scale / 100))
+        self.darea.set_size_request(w, h)
+        self.base_x, self.base_y = self.get_coordinate(w, h)
+
+    def set_layout(self):
+        if self.window is None:
+            return
+        if self.layout is None:
+            return
+        font_size = self.font_size * 3 // 4 # convert from Windows to GTK+
+        font_size = font_size * Pango.SCALE
+        self.font_desc.set_size(font_size)
+        self.layout.set_font_description(self.font_desc)
+        self.layout.set_width(int((self.right - self.left) * 1024))
+
+    def reset_scale(self):
+        if self.window is None:
+            return
+        self.set_skin()
+        self.set_position()
+        self.darea.queue_draw()
+
+    @property
     def clickerase(self):
         return self.data.get('clickerase', 'on') == 'on'
 
+    @property
     def dragmove_horizontal(self):
         return self.data.get('dragmove.horizontal') == 'on'
 
+    @property
     def dragmove_vertical(self):
         return self.data.get('dragmove.vertical') == 'on'
 
@@ -336,17 +381,17 @@ class Balloon:
             x = left + scrn_w - w
             y = top + scrn_h - h
         elif self.position == 'center':
-            x = left + (scrn_w - w) / 2
-            y = top + (scrn_h - h) / 2
+            x = left + (scrn_w - w) // 2
+            y = top + (scrn_h - h) // 2
         elif self.position == 'leftcenter':
-            y = top + (scrn_h - h) / 2
+            y = top + (scrn_h - h) // 2
         elif self.position == 'rightcenter':
             x = left + scrn_w - w
-            y = top + (scrn_h - h) / 2
+            y = top + (scrn_h - h) // 2
         elif self.position == 'centertop':
-            x = left + (scrn_w - w) / 2
+            x = left + (scrn_w - w) // 2
         elif self.position == 'centerbottom':
-            x = left + (scrn_w - w) / 2
+            x = left + (scrn_w - w) // 2
             y = top + scrn_h - h
         elif self.position == 'sakura':
             if self.direction: # right
@@ -369,6 +414,8 @@ class Balloon:
         return x, y
 
     def update_script(self, text, mode):
+        if self.timeout_id is None: # XXX
+            return
         if not text:
             return
         if mode == 2 and self.script is not None:
@@ -382,11 +429,11 @@ class Balloon:
         self.quick_session = 0
         try:
             self.processed_script = self.script_parser.parse(self.script)
-        except ninix.script.ParserError, e:
+        except ninix.script.ParserError as e:
             self.processed_script = None
-            print '-' * 50
-            print e
-            print self.script                                    
+            logging.error('-' * 50)
+            logging.error(e)
+            logging.error(self.script.encode('utf-8'))
 
     def get_sakura_is_talking(self):
         talking = 0
@@ -491,46 +538,44 @@ class Balloon:
                                                 % self.action['ref2'])
                                         / self.action['ref2'])
                     if self.action['ref0']:
-                        self.action_y = int(offset * self.scale / 100)
+                        self.action_y = int(offset)
                     else:
-                        self.action_x = int(offset * self.scale / 100)
+                        self.action_x = int(offset)
                 elif self.action['method'] == 'vibrate':
-                    offset = (int((time.time() - self.start_time) * 1000) / \
+                    offset = (int((time.time() - self.start_time) * 1000) // \
                               self.action['ref2']) % 2
-                    self.action_x = int(offset * self.action['ref0'] * self.scale / 100)
-                    self.action_y = int(offset * self.action['ref1'] * self.scale / 100)
+                    self.action_x = int(offset * self.action['ref0'])
+                    self.action_y = int(offset * self.action['ref1'])
             if (self.slide_vx != 0 or self.slide_vy != 0) and \
                    self.slide_autostop > 0 and \
                    self.slide_autostop * 0.001 + 0.05 <= time.time() - self.start_time:
-                self.vx = self.direction * int((self.slide_autostop / 50.0 + 1) * self.slide_vx * self.scale / 100)
+                self.vx = self.direction * int((self.slide_autostop / 50.0 + 1) * self.slide_vx)
                 self.slide_vx = 0
-                self.vy = int((self.slide_autostop / 50.0 + 1) * self.slide_vy * self.scale / 100)
+                self.vy = int((self.slide_autostop / 50.0 + 1) * self.slide_vy)
                 self.slide_vy = 0
             if self.slide_vx != 0:
-                self.vx = self.direction * int(((time.time() - self.start_time) * self.slide_vx * self.scale / 100) / 50 * 1000.)
+                self.vx = self.direction * int(((time.time() - self.start_time) * self.slide_vx) / 50 * 1000.)
             if self.slide_vy != 0:
-                self.vy = int(((time.time() - self.start_time) * self.slide_vy * self.scale / 100) / 50 * 1000.)
-            self.window.move(int(self.x + self.action_x + self.vx),
-                             int(self.y + self.action_y + self.vy))
+                self.vy = int(((time.time() - self.start_time) * self.slide_vy) / 50 * 1000.)
+            self.set_position()
             if self.processed_script or self.processed_text:
                 self.interpret_script()
         self.talking = 0 if self.talking and not sakura_talking else sakura_talking
         return True
 
-    def redraw(self, darea, event):
-        cr = darea.window.cairo_create()
-        cr.save()
-        cr.set_operator(cairo.OPERATOR_CLEAR)
-        cr.paint()
-        cr.restore()
-        cr.set_source_pixbuf(self.current_balloon_pixbuf, 0, 0)
-#        cr.paint_with_alpha(self.__alpha_channel)
+    def redraw(self, widget, cr):
+        scale = self.scale
+        cr.scale(scale / 100.0, scale / 100.0)
+        cr.set_source_surface(self.balloon_surface, 0, 0)
+        cr.set_operator(cairo.OPERATOR_SOURCE)
         cr.paint()
+        cr.set_operator(cairo.OPERATOR_OVER) # restore default
+        self.window.update_shape(cr.get_target())
         if self.layout:
             cr.set_source_rgb(*self.fontcolor)
-            cr.move_to(self.left, self.top)
-            cr.show_layout(self.layout)
-        del cr
+            cr.move_to(int(self.left), int(self.top))
+            PangoCairo.update_layout(cr, self.layout)
+            PangoCairo.show_layout(cr, self.layout)
 
     def get_state(self):
         return self.state
@@ -591,17 +636,17 @@ class Balloon:
             self.processed_text = text
 
     def draw_text(self, text):
-        self.layout.set_text(text)
+        self.layout.set_text(text, -1)
         self.darea.queue_draw()
 
     def button_press(self, widget, event):
         self.x_root = event.x_root
         self.y_root = event.y_root
-        if event.type == gtk.gdk._2BUTTON_PRESS:
+        if event.type == Gdk.EventType._2BUTTON_PRESS:
             x = int(event.x * 100 / self.scale)
             y = int(event.y * 100 / self.scale)
-            self.__sakura.notify_event('OnEBMouseDoubleClick', self.name,
-                                       x, y, self.id)
+            self.__sakura.notify_event(
+                'OnEBMouseDoubleClick', self.name, x, y, self.id)
         return True
 
     def button_release(self, widget, event):
@@ -609,37 +654,37 @@ class Balloon:
         self.y_root = None
         x = int(event.x * 100 / self.scale)
         y = int(event.y * 100 / self.scale)
-        if event.type == gtk.gdk.BUTTON_RELEASE:
+        if event.type == Gdk.EventType.BUTTON_RELEASE:
             if event.button == 1:
-                self.__sakura.notify_event('OnEBMouseClick', self.name,
-                                           x, y, self.id, 0)
+                self.__sakura.notify_event(
+                    'OnEBMouseClick', self.name, x, y, self.id, 0)
             elif event.button == 3:
-                self.__sakura.notify_event('OnEBMouseClick', self.name,
-                                           x, y, self.id, 1)
-        if self.clickerase():
+                self.__sakura.notify_event(
+                    'OnEBMouseClick', self.name, x, y, self.id, 1)
+        if self.clickerase:
             self.destroy()
         return True
 
     def motion_notify(self, widget, event):
+        scale = self.scale
         if self.x_root is not None and \
            self.y_root is not None:
             x_delta = int(event.x_root - self.x_root)
             y_delta = int(event.y_root - self.y_root)
-            if event.state & gtk.gdk.BUTTON1_MASK:
-                if self.dragmove_horizontal():
+            if event.get_state() & Gdk.ModifierType.BUTTON1_MASK:
+                if self.dragmove_horizontal:
                     self.x += x_delta
-                if self.dragmove_vertical():
+                if self.dragmove_vertical:
                     self.y += y_delta
-                self.window.move(int(self.x + self.action_x + self.vx),
-                                 int(self.y + self.action_y + self.vy))
+                self.set_position()
                 self.x_root = event.x_root
                 self.y_root = event.y_root
         if self.move_notify_time is None or \
            time.time() - self.move_notify_time > 500 * 0.001:
-            x = int(event.x * 100 / self.scale)
-            y = int(event.y * 100 / self.scale)
-            self.__sakura.notify_event('OnEBMouseMove', self.name,
-                                       x, y, self.id)
+            x = int(event.x * 100 / scale)
+            y = int(event.y * 100 / scale)
+            self.__sakura.notify_event(
+                'OnEBMouseMove', self.name, x, y, self.id)
             self.move_notify_time = time.time()
         return True
 
@@ -655,5 +700,5 @@ class Balloon:
             self.window.destroy()
             self.window = None
         if self.timeout_id:
-            glib.source_remove(self.timeout_id)
+            GLib.source_remove(self.timeout_id)
             self.timeout_id = None
