@@ -177,10 +177,10 @@ class Installer:
         filetype = self.get_file_type(tmpdir)
         try:
             func = getattr(self, 'install_{0}'.format(filetype))
-            target_dir = func(filename, tmpdir, homedir)
+            target_dirs = func(filename, tmpdir, homedir)
         finally:
             shutil.rmtree(tmpdir)
-        return filetype, target_dir
+        return filetype, target_dirs
 
 
     def download(self, url, basedir):
@@ -242,8 +242,8 @@ class Installer:
         path = os.path.abspath(target_dir)
         if not os.path.isdir(path):
             return
-        os.path.walk(path, self.remove_files, mask)
-        dirlist = self.list_all_directories(path, '')
+        os.walk(path, self.remove_files, mask)
+        dirlist = self.list_all_directories(path, b'')
         dirlist.sort()
         dirlist.reverse()
         for name in dirlist:
@@ -361,7 +361,7 @@ class Installer:
                     # uninstall older versions of the balloon
                     self.remove_files_and_dirs(balloon_dst, [])
                 balloon_list = []
-                for path in self.list_all_files(os.path.join(tmpdir, balloon_src), ''):
+                for path in self.list_all_files(os.path.join(tmpdir, balloon_src), b''):
                     balloon_list.append((os.path.join(tmpdir, balloon_src, path),
                                          os.path.join(balloon_dst, path)))
                 self.install_files(balloon_list)
@@ -392,7 +392,9 @@ class Installer:
             except IOError as e:
                 code, message = e.args
                 logging.error('cannot write {0}'.format(path))
-        return os.fsdecode(target_dir)
+        if balloon_dir is None: # XXX
+            balloon_dir = ''
+        return (os.fsdecode(target_dir), os.fsdecode(balloon_dir))
 
     def install_supplement(self, archive, tmpdir, homedir):
         inst = ninix.home.read_install_txt(tmpdir)
