@@ -420,17 +420,37 @@ class Menu:
                         if len(entry) < 2:
                             continue
                         url = entry[1]
-                        ## FIXME
-                        #if len(entry) > 2:
-                        #    bannar = entry[2]
-                        #else:
-                        #    bannar = None
+                        if len(entry) > 2:
+                            base_path = self.request_parent(
+                                'GET', 'get_prefix')
+                            filename = entry[2].lower()
+                            head, tail = os.path.splitext(filename)
+                            if not tail:
+                                for ext in ['.png', '.jpg', '.gif']:
+                                    filename = ''.join((filename, ext))
+                                    banner = os.path.join(
+                                        base_path, b'ghost/master/banner',
+                                        os.fsencode(filename))
+                                    if not os.path.exists(banner):
+                                        banner = None
+                                    else:
+                                        break
+                            else:
+                                banner = os.path.join(
+                                    base_path, b'ghost/master/banner',
+                                    os.fsencode(filename))
+                                if not os.path.exists(banner):
+                                    banner = None
+                        else:
+                            banner = None
                         item = Gtk.MenuItem(title)
                         item.connect(
                             'activate',
                             lambda a, i: self.request_parent(
                                 'NOTIFY', 'notify_site_selection', i),
                             (title, url))
+                        item.set_has_tooltip(True)
+                        item.connect('query-tooltip', self.on_tooltip, banner)
                     menu.add(item)
                     item.show()
                 menuitem = self.ui_manager.get_widget(
@@ -456,17 +476,36 @@ class Menu:
                     if len(entry) < 2:
                         continue
                     url = entry[1]
-                    ## FIXME
-                    #if len(entry) > 2:
-                    #    bannar = entry[2]
-                    #else:
-                    #    bannar = None
+                    if len(entry) > 2:
+                        base_path = self.request_parent('GET', 'get_prefix')
+                        filename = entry[2].lower()
+                        head, tail = os.path.splitext(filename)
+                        if not tail:
+                            for ext in ['.png', '.jpg', '.gif']:
+                                filename = ''.join((filename, ext))
+                                banner = os.path.join(
+                                    base_path, b'ghost/master/banner',
+                                    os.fsencode(filename))
+                                if not os.path.exists(banner):
+                                    banner = None
+                                else:
+                                    break
+                        else:
+                            banner = os.path.join(
+                                base_path, b'ghost/master/banner',
+                                os.fsencode(filename))
+                            if not os.path.exists(banner):
+                                banner = None
+                    else:
+                        banner = None
                     item = Gtk.MenuItem(title)
                     item.connect(
                         'activate',
                         lambda a, i: self.request_parent(
                             'NOTIFY', 'notify_site_selection', i),
                         (title, url))
+                    item.set_has_tooltip(True)
+                    item.connect('query-tooltip', self.on_tooltip, banner)
                 menu.add(item)
                 item.show()
             menuitem =  self.ui_manager.get_widget(
@@ -477,7 +516,7 @@ class Menu:
         else:
             self.__set_visible('Recommend', 0)
 
-    def create_ghost_menuitem(self, name, icon, key, handler):
+    def create_ghost_menuitem(self, name, icon, key, handler, thumbnail):
         if icon is not None:
             pixbuf = ninix.pix.create_icon_pixbuf(icon)
             if pixbuf is None:
@@ -494,8 +533,17 @@ class Menu:
         item.set_name('popup menu item')
         item.show()
         item.connect('activate', lambda a, v: handler(v), (key))
+        item.set_has_tooltip(True)
+        item.connect('query-tooltip', self.on_tooltip, thumbnail)
         return item
 
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip, thumbnail):
+        if thumbnail is None:
+            return False
+        pixbuf = ninix.pix.create_pixbuf_from_file(thumbnail)
+        tooltip.set_icon(pixbuf)
+        return True
+
     def __set_ghost_menu(self):
         for path in ['Summon', 'Change']:
             ghost_menu = Gtk.Menu()
@@ -527,11 +575,13 @@ class Menu:
                 menu.append(item)
         return menu
 
-    def create_meme_menuitem(self, name, value, handler):
+    def create_meme_menuitem(self, name, value, handler, thumbnail):
         item = Gtk.MenuItem(name)
         item.set_name('popup menu item')
         item.show()
         item.connect('activate', lambda a, v: handler(v), (value))
+        item.set_has_tooltip(True)
+        item.connect('query-tooltip', self.on_tooltip, thumbnail)
         return item
 
     def __set_plugin_menu(self):
