@@ -60,8 +60,15 @@ class ShellMeme(Meme):
 
     def create_menuitem(self, data):
         shell_name = data[0]
+        subdir = data[1]
+        base_path = self.request_parent('GET', 'get_prefix')
+        thumbnail_path = os.path.join(base_path, b'shell',
+                                      subdir, b'thumbnail.png')
+        if not os.path.exists(thumbnail_path):
+            thumbnail_path = None
         return self.request_parent(
-            'GET', 'create_shell_menuitem', shell_name, self.key)
+            'GET', 'create_shell_menuitem', shell_name, self.key,
+            thumbnail_path)
 
     def delete_by_myself(self):
         self.request_parent('NOTIFY', 'delete_shell', self.key)
@@ -199,6 +206,7 @@ class Sakura:
         self.desc = desc
         self.shiori_dir = shiori_dir
         self.use_makoto = use_makoto
+        self.prefix = prefix
         self.shells = OrderedDict()
         shell_menuitems = OrderedDict()
         for key, value in surface_set.items():
@@ -209,7 +217,6 @@ class Sakura:
             shell_menuitems[key] = meme.menuitem
         self.shell_menu = self.request_parent(
             'GET', 'create_shell_menu', shell_menuitems)
-        self.prefix = prefix
         self.shiori_dll = shiori_dll
         self.shiori_name = shiori_name
         name = (shiori_dll, shiori_name)
@@ -1788,6 +1795,8 @@ class Sakura:
         args = [self.expand_meta(s) for s in args]
         if args[0] == 'raise' and argc >= 2:
             self.notify_event(*args[1:10])
+        elif args[0:2] == ['open', 'readme']:
+            ReadmeDialog().show(self.get_name(), self.get_prefix())
         elif args[0:2] == ['open', 'browser'] and argc > 2:
             webbrowser.open(args[2])
         elif args[0:2] == ['open', 'communicatebox']:
@@ -2317,6 +2326,55 @@ class VanishDialog:
         return True
 
 
+class ReadmeDialog:
+
+    def __init__(self):
+        self.request_parent = lambda *a: None # dummy
+        self.dialog = Gtk.Dialog()
+        self.dialog.connect('delete_event', lambda *a: True) # XXX
+        self.dialog.set_title('Readme.txt')
+        self.dialog.set_modal(False)
+        self.dialog.set_resizable(False)
+        self.dialog.set_position(Gtk.WindowPosition.CENTER)
+        self.label = Gtk.Label()
+        self.label.show()
+        self.textview = Gtk.TextView()
+        self.textview.set_editable(False)
+        self.textview.set_cursor_visible(False)
+        self.textview.show()
+        scroll = Gtk.ScrolledWindow(None, None)
+        scroll.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        scroll.add(self.textview)
+        scroll.show()
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        vbox.set_size_request(720, 460)
+        vbox.show()
+        vbox.pack_start(self.label, False, True, 0)
+        vbox.pack_start(scroll, True, True, 0)
+        content_area = self.dialog.get_content_area()
+        content_area.add(vbox)
+        self.dialog.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
+        self.dialog.connect('response', self.response)
+
+    def set_responsible(self, request_method):
+        self.request_parent = request_method
+
+    def show(self, name, base_path):
+        self.label.set_text(name)
+        path = os.path.join(base_path, b'readme.txt')
+        if os.path.exists(path):
+            with open(path, encoding='CP932') as f: # XXX
+                text = f.read()
+                self.textview.get_buffer().set_text(text)
+                self.dialog.show()
+
+    def response(self, widget, response):
+        func = {Gtk.ResponseType.CLOSE: widget.hide,
+                Gtk.ResponseType.DELETE_EVENT: widget.hide,
+                }
+        func[response]()
+        return True
+
 def test():
     pass
 
