@@ -2,7 +2,7 @@
 #
 #  script.py - a Sakura Script parser
 #  Copyright (C) 2001, 2002 by Tamito KAJIYAMA
-#  Copyright (C) 2004-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
+#  Copyright (C) 2004-2013 by Shyouzou Sugitani <shy@users.sourceforge.jp>
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License (version 2) as
@@ -24,7 +24,7 @@ TOKEN_NUMBER      = 5
 TOKEN_STRING      = 6
 
 patterns = [
-    (TOKEN_TAG, re.compile(r'\\[ehunjcxtqzy*v0123456789fmia!&+---]|'
+    (TOKEN_TAG, re.compile(r'\\[Cehunjcxtqzy*v0123456789fmia!&+---]|'
                            r'\\[sbp][0-9]?|\\w[0-9]|\\_[wqslvVbe+cumna]|'
                            r'\\__[ct]|\\URL')),
     (TOKEN_META, re.compile(r'%month|%day|%hour|%minute|%second|%username|'
@@ -199,7 +199,8 @@ class Parser:
                           '\\v', '\\x', '\\y', '\\z', '\\_q',
                           '\\4', '\\5', '\\6', '\\7',
                           '\\2', '\\*', '\\-', '\\+', '\\_+',
-                          '\\_n', '\\_V', '\\__c', '\\__t']:
+                          '\\_n', '\\_V', '\\__c', '\\__t',
+                          '\\C']:
                 self.script.append((SCRIPT_TAG, lexeme, self.column))
             elif lexeme in ['\\0', '\\h']:
                 self.script.append((SCRIPT_TAG, lexeme, self.column))
