@@ -502,8 +502,13 @@ class BalloonMeme(Meme):
         desc, balloon = data
         subdir = balloon['balloon_dir'][0]
         name = desc.get('name', subdir)
+        home_dir = ninix.home.get_ninix_home()
+        thumbnail_path = os.path.join(home_dir, b'balloon',
+                                      subdir, b'thumbnail.png')
+        if not os.path.exists(thumbnail_path):
+            thumbnail_path = None
         return self.request_parent(
-            'GET', 'create_balloon_menuitem', name, self.key)
+            'GET', 'create_balloon_menuitem', name, self.key, thumbnail_path)
 
     def delete_by_myself(self):
         self.request_parent('NOTIFY', 'delete_balloon', self.key)
@@ -588,6 +593,7 @@ class Application:
             'notify_other': self.communicate.notify_other,
             'reset_sstp_flag': self.sstp_controler.reset_sstp_flag,
             'get_sstp_port': self.sstp_controler.get_sstp_port,
+            'get_prefix': self.get_sakura_prefix,
             }
         handler = handlers.get(event,
                                getattr(self, event,
@@ -604,17 +610,31 @@ class Application:
 
     def do_install(self, filename):
         try:
-            filetype, target_dir = self.installer.install(
+            filetype, target_dirs = self.installer.install(
                 filename, ninix.home.get_ninix_home())
         except:
-            target_dir = None
-        if target_dir is not None:
+            target_dirs = None
+        if target_dirs:
             if filetype == 'ghost':
-                self.add_sakura(target_dir)
+                self.add_sakura(target_dirs[0])
+                ninix.sakura.ReadmeDialog().show(
+                    target_dirs[0],
+                    os.path.join(ninix.home.get_ninix_home(),
+                                 b'ghost', os.fsencode(target_dirs[0])))
+                if target_dirs[1]:
+                    self.add_balloon(target_dirs[1])
+                    ninix.sakura.ReadmeDialog().show(
+                        target_dirs[1],
+                        os.path.join(ninix.home.get_ninix_home(),
+                                     b'balloon', os.fsencode(target_dirs[1])))
             elif filetype == 'supplement':
-                self.add_sakura(target_dir) # XXX: reload
+                self.add_sakura(target_dirs) # XXX: reload
             elif filetype == 'balloon':
-                self.add_balloon(target_dir)
+                self.add_balloon(target_dirs)
+                ninix.sakura.ReadmeDialog().show(
+                    target_dirs,
+                    os.path.join(ninix.home.get_ninix_home(),
+                                 b'balloon', os.fsencode(target_dirs)))
             elif filetype == 'plugin':
                 self.plugins = ninix.home.search_plugins()
             elif filetype == 'nekoninni':
@@ -671,6 +691,9 @@ class Application:
                 continue
             yield sakura
 
+    def get_sakura_prefix(self):
+        return self.__menu_owner.get_prefix()
+
     def getstring(self, name):
         return self.__menu_owner.getstring(name)
 
@@ -748,9 +771,9 @@ class Application:
             menuitem.set_sensitive(key != current_key) # not working
         return self.balloon_menu
 
-    def create_balloon_menuitem(self, balloon_name, balloon_key):
+    def create_balloon_menuitem(self, balloon_name, balloon_key, thumbnail):
         return self.__menu.create_meme_menuitem(
-            balloon_name, balloon_key, self.select_balloon)
+            balloon_name, balloon_key, self.select_balloon, thumbnail)
 
     def create_balloon_menu(self):
         balloon_menuitems = OrderedDict()
@@ -761,9 +784,9 @@ class Application:
     def create_shell_menu(self, menuitems):
         return self.__menu.create_meme_menu(menuitems)
 
-    def create_shell_menuitem(self, shell_name, shell_key):
+    def create_shell_menuitem(self, shell_name, shell_key, thumbnail):
         return self.__menu.create_meme_menuitem(
-            shell_name, shell_key, self.select_shell)
+            shell_name, shell_key, self.select_shell, thumbnail)
 
     def create_menuitem(self, key, baseinfo):
         desc = baseinfo[0]
@@ -777,10 +800,15 @@ class Application:
         else:
             icon_path = None
         name = desc.get('name')
+        thumbnail_path = os.path.join(shiori_dir, b'thumbnail.png')
+        if not os.path.exists(thumbnail_path):
+            thumbnail_path = None
         start_menuitem = self.__menu.create_ghost_menuitem(
-            name, icon_path, key, self.start_sakura_cb) # XXX
+            name, icon_path, key, self.start_sakura_cb, # XXX
+            thumbnail_path)
         select_menuitem = self.__menu.create_ghost_menuitem(
-            name, icon_path, key, self.select_sakura)
+            name, icon_path, key, self.select_sakura,
+            thumbnail_path)
         menuitem = {
             'Summon': start_menuitem,
             'Change': select_menuitem,
